import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "../components"
import "view"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Форма просмотра информации о карте
 */
BasicView {
    readonly property var cardRecord: Utils.variantToObjectPointer(params.card)
    readonly property var client: cardRecord.card.client
    readonly property bool fullScreen: true

    implicitWidth: screenWidth
    implicitHeight: screenHeight

    component InfoBox: GroupBox {
        Style.class: "cardInfoCommonData"
        interactive: false
        leftPadding: 32
        topPadding: 24
        rightPadding: 32
        bottomPadding: 24
    }

    contentItem: RowLayout {
        spacing: 0
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            Flickable {
                anchors.fill: parent
                contentHeight: content.implicitHeight
                ColumnLayout {
                    id: content
                    anchors {
                        left: parent.left; right: parent.right; top: parent.top
                        leftMargin: 24; topMargin: 16; rightMargin: 24
                    }
                    spacing: 16
                    BasicLabel {
                        Style.class: "cardInfoTitle"
                        Tr {
                            code: "cardInfoFormTitle"
                            defaultMessage: "Информация о карте № %1"
                            args: [ cardRecord.number ]
                        }
                    }
                    Item { Layout.preferredHeight: 16 }
                    RowLayout {
                        spacing: 16
                        GroupBox {
                            Style.class: "cardInfoMainData"
                            Layout.preferredWidth: 232
                            Layout.preferredHeight: clientInfo.implicitHeight
                            interactive: false
                            ColumnLayout {
                                anchors {
                                    top: parent.top; right: parent.right; left: parent.left
                                    topMargin: 24; leftMargin: 32; rightMargin: 32
                                }
                                BasicLabel {
                                    Style.class: "cardInfoCommonLabel"
                                    Tr { code: "cardInfoFormBonusBalance"; defaultMessage: "Баланс" }
                                }
                                BasicLabel {
                                    Style.class: "cardInfoBalanceValue"
                                    text: Utils.formatMoneyItem(cardRecord.bonusBalance)
                                }
                            }
                        }
                        InfoBox {
                            id: clientInfo
                            Layout.fillWidth: true
                            contentItem: ColumnLayout {
                                BasicLabel {
                                    Style.class: "cardInfoCommonLabel"
                                    Tr { code: "cardInfoFormClientName"; defaultMessage: "Клиент" }
                                }
                                BasicLabel {
                                    Style.class: "cardInfoCommonValue"
                                    Layout.fillWidth: true
                                    text: client ? client.name : ""
                                }
                                BasicLabel {
                                    Style.class: "cardInfoCommonLabel"
                                    Tr { code: "cardInfoFormPhoneNumber"; defaultMessage: "Телефон" }
                                }
                                BasicLabel {
                                    Style.class: "cardInfoCommonValue"
                                    Layout.fillWidth: true
                                    text: client ? client.phoneNumber : ""
                                }
                                BasicLabel {
                                    Style.class: "cardInfoCommonLabel"
                                    Tr { code: "cardInfoFormClientEmail"; defaultMessage: "Email" }
                                }
                                BasicLabel {
                                    Style.class: "cardInfoCommonValue"
                                    Layout.fillWidth: true
                                    text: client ? client.email : ""
                                }
                            }
                        }
                    }
                    InfoBox {
                        id: countersBox

                        readonly property list<real> columnsWidth: [tableView.width - 250, 250]
                        readonly property list<var> columnNames: [ Utils.makeTr("cardInfoFormCountersName", "Счетчик"),
                                Utils.makeTr("cardInfoFormCountersSum", "Сумма") ]
                        ColumnSortingProxyModel {
                            id: proxyModel
                            sourceModel: CounterModel { cardRecord: params.card }
                            columnsOrder: [ "countername", "countersum" ]
                        }
                        Layout.fillWidth: true
                        visible: tableView.rows > 0
                        contentItem: ColumnLayout {
                            spacing: 0
                            BasicLabel {
                                Style.class: "cardInfoTableLabel"
                                Tr { code: "cardInfoFormCountersTitle"; defaultMessage: "Счетчики" }
                            }
                            Item { Layout.preferredHeight: 8 }
                            HorizontalHeaderView {
                                id: headerView
                                Layout.fillWidth: true
                                Layout.preferredHeight: 30
                                model: proxyModel
                                clip: true
                                resizableColumns: false
                                rowHeightProvider: function(row) { return 30 }
                                columnWidthProvider: function(column) { return countersBox.columnsWidth[column] }
                                interactive: false
                                delegate: BasicLabel {
                                    Style.class: "cardInfoColumnLabel"
                                    padding: 0
                                    leftPadding: column === 0 ? 24 : 0
                                    rightPadding: column === 1 ? 24 : 0
                                    verticalAlignment: Label.AlignVCenter
                                    Tr { tr: countersBox.columnNames[column] }
                                }

                                // FIXME: модель шлет сигнал modelReset, из-за чего виджет пересоздается.
                                // Но после удаления старый виджет продолжает отображаться на экране, причем в неправильной позиции.
                                // Из-за этого возникает задвоение заголовков. Установка `visible = false` при удалении - костыль
                                Component.onDestruction: () => { visible = false }
                            }
                            Line {
                                Layout.fillWidth: true
                                Layout.preferredHeight: strokeWidth
                                strokeWidth: 2
                                color: Style.parameter("color.grayDark")
                            }
                            TableView {
                                id: tableView
                                Layout.fillWidth: true
                                Layout.preferredHeight: rows * 52 + (rows - 1)
                                rowSpacing: 1
                                columnWidthProvider: function(column) { return countersBox.columnsWidth[column] }
                                rowHeightProvider: function(row) { return 52 }
                                model: proxyModel
                                delegate: BasicLabel {
                                    Style.class: "cardInfoTableValue"
                                    leftPadding: column === 0 ? 24 : 0
                                    rightPadding: column === 1 ? 24 : 0
                                    verticalAlignment: Label.AlignVCenter
                                    text: display
                                    Line {
                                        anchors.top: parent.bottom
                                        width: parent.width
                                        strokeWidth: 1
                                        color: Style.parameter("color.grayBorder")
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 8
                BasicButton {
                    Style.class: "cardInfoCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "cardInfoFormCancel"; defaultMessage: "Закрыть" }
                    onClicked: handler.sendAnswer()
                }
            }
        }
    }

    footer: BottomPanel {}

    DialogHandler { id: handler }
}
