import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes
import Qt.labs.qmlmodels

import "../components"
import "./view"

import Artix.Core
import Artix.Gui
import Artix.Models
import Artix.Handlers

/**
 * @brief Диалог подбора товара
 */
BasicView {
    id: root
    readonly property bool fullScreen: true

    property BasicTextField eventReceiver: inputField
    property DialogEventFilter eventFilter: DialogEventFilter {
        onOk: {
            if (selectionModel.hasSelection)
                accept()
            keyboard.close()
        }
        onCancel: {
            if (inputField.focus)
                inputField.focus = false
            else
                reject()
        }
    }

    property string nameFilter: ""
    property string barcodeFilter: ""
    property string codeFilter: ""
    property string priceFilter: ""
    onNameFilterChanged: updateModel()
    onBarcodeFilterChanged: updateModel()
    onCodeFilterChanged: updateModel()
    onPriceFilterChanged: updateModel()

    implicitWidth: screenWidth
    implicitHeight: screenHeight
    topPadding: 16

    contentItem: ColumnLayout {
        spacing: 0
        BasicLabel {
            Style.class: "tmcMultipleChoiceTitle"
            Layout.leftMargin: 24
            Layout.rightMargin: 24
            Tr { code: "tmcMultipleChoiceTitle"; defaultMessage: "Поиск товара" }
        }
        Item { Layout.preferredHeight: 16 }
        GroupBox {
            Style.class: "tmcMultipleChoiceField"
            Layout.fillWidth: true
            Layout.preferredHeight: 64
            Layout.leftMargin: 24
            Layout.rightMargin: 24
            interactive: false
            leftPadding: 12
            rightPadding: 12
            contentItem: RowLayout {
                spacing: 0
                BasicTextField {
                    id: inputField
                    Style.class: "tmcMultipleChoiceInput"
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    Layout.topMargin: 12
                    Layout.bottomMargin: 12
                    rightPadding: 34
                    Image {
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        width: 30
                        height: 30
                        source: Style.parameter("icon.textFieldClear")
                        MouseArea { anchors.fill: parent; onClicked: inputField.clear() }
                    }
                    DoubleValidator {
                        id: dv
                        notation: DoubleValidator.StandardNotation
                        bottom: 0.0
                        decimals: 2
                    }
                    validator: tabBar.currentIndex === 3 ? dv : null
                    Timer {
                        id: inputTimer
                        interval: 1000
                        onTriggered: {
                            switch (tabBar.currentIndex) {
                            case 0:
                                if (nameFilter !== inputField.text)
                                    nameFilter = inputField.text
                                break
                            case 1:
                                if (barcodeFilter !== inputField.text)
                                    barcodeFilter = inputField.text
                                break
                            case 2:
                                if (codeFilter !== inputField.text)
                                    codeFilter = inputField.text
                                break
                            case 3:
                                if (priceFilter !== inputField.text)
                                    priceFilter = inputField.text
                                break
                            }
                        }
                    }
                    onTextChanged: inputTimer.restart()
                }
                Item { Layout.preferredWidth: 4 }
                TabBar {
                    id: tabBar
                    Layout.fillHeight: true
                    Layout.preferredWidth: 470
                    Layout.topMargin: 8
                    Layout.bottomMargin: 8
                    padding: 0
                    background: Item {}
                    contentWidth: width
                    contentHeight: height
                    component FieldTab: TabButton {
                        id: tab
                        Style.type: "button"
                        Style.class: "tmcMultipleChoiceTab"
                        font: tab.Style.text.font
                        palette.buttonText: tab.Style.text.color
                        focusPolicy: Qt.NoFocus
                        background: Rectangle {
                            color: tab.Style.background.color
                            border.color: tab.Style.background.border.color
                            border.width: tab.Style.background.border.width
                            radius: tab.Style.background.radius

                            Behavior on color { ColorAnimation { duration: 100 }}
                        }
                        property alias filterIconVisible: filterIcon.visible
                        Image {
                            id: filterIcon
                            anchors { top: parent.top; right: parent.right; margins: 8 }
                            source: Style.parameter("icon.filter")
                            height: 16; width: 16
                            fillMode: Image.PreserveAspectFit
                        }
                    }
                    FieldTab {
                        width: 160
                        down: TabBar.tabBar.currentIndex == TabBar.index
                        filterIconVisible: nameFilter.length > 0
                        Tr { code: "tmcMultipleChoiceNameTab"; defaultMessage: "Наименование" }
                        onClicked: inputField.text = nameFilter
                    }
                    FieldTab {
                        width: 140
                        down: TabBar.tabBar.currentIndex == TabBar.index
                        filterIconVisible: barcodeFilter.length > 0
                        Tr { code: "tmcMultipleChoiceBarcodeTab"; defaultMessage: "Штрих-код" }
                        onClicked: inputField.text = barcodeFilter
                    }
                    FieldTab {
                        width: 80
                        down: TabBar.tabBar.currentIndex == TabBar.index
                        filterIconVisible: codeFilter.length > 0
                        Tr { code: "tmcMultipleChoiceCodeTab"; defaultMessage: "Код" }
                        onClicked: inputField.text = codeFilter
                    }
                    FieldTab {
                        width: 90
                        down: TabBar.tabBar.currentIndex == TabBar.index
                        filterIconVisible: priceFilter.length > 0
                        Tr { code: "tmcMultipleChoicePriceTab"; defaultMessage: "Цена" }
                        onClicked: inputField.text = priceFilter
                    }
                }
            }
        }
        Item { Layout.preferredHeight: 16 }
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            TableView {
                id: tableView
                anchors.fill: parent
                signal rowPressedChanged(int row, bool value)
                clip: true
                rowSpacing: 1
                columnWidthProvider: function(column) {
                    switch (column) {
                    case 0: return width - 480
                    case 1: return 180
                    case 2: return 100
                    case 3: return 200
                    }
                }
                readonly property real delegateHeight: 52
                rowHeightProvider: function(row) { return delegateHeight }
                model: ColumnSortingProxyModel {
                    id: proxyModel
                    sourceModel: TmcMultipleChoiceModel { id: dataModel }
                    columnsOrder: [
                        "name",
                        "barcode",
                        "code",
                        "price"
                    ]
                }
                selectionModel: ItemSelectionModel { id: selectionModel; model: proxyModel }
                delegate: BasicButton {
                    id: tableDelegate
                    Style.class: "tmcMultipleChoiceDelegate"
                    textLayout { fillWidth: true; fillHeight: true }
                    leftPadding: column === 0 ? 24 : 0
                    rightPadding: column === 3 ? 40 : 0
                    text: column === 3 ? Utils.formatMoneyItem(display) : display
                    verticalAlignment: Label.AlignVCenter
                    horizontalAlignment: column === 3 ? Label.AlignRight : Label.AlignLeft
                    pressed: false
                    down: pressed || selectionModel.currentIndex.row === row
                    Line {
                        anchors.top: parent.bottom
                        width: tableView.width
                        strokeWidth: 1
                        color: Style.parameter("color.grayBorder")
                    }
                    MouseArea {
                        id: ma
                        anchors.fill: parent
                        onPressedChanged: tableView.rowPressedChanged(row, pressed)
                        onClicked: {
                            inputField.focus = false
                            selectionModel.setCurrentIndex(proxyModel.index(row, column), ItemSelectionModel.SelectCurrent) // выделить позицию
                        }
                    }
                    Connections {
                        target: tableView
                        function onRowPressedChanged(row, pressed) {
                            if (row === model.row)
                                tableDelegate.pressed = pressed
                        }
                    }
                }
                // Дозагрузка данных при скролинге
                onMovementEnded: {
                    if (atYBeginning) {
                        if (dataModel.top === 0)
                            return
                        let rowsShift = dataModel.count / 2
                        dataModel.top -= rowsShift
                        updateModel()
                        forceLayout()
                        positionViewAtRow(rowsShift, TableView.AlignBottom)
                    }
                    if (atYEnd) {
                        if (dataModel.rowCount() < dataModel.count)
                            return
                        let rowsShift = dataModel.count / 2
                        dataModel.top += rowsShift
                        updateModel()
                        forceLayout()
                        positionViewAtRow(rowsShift - 1, TableView.AlignTop)
                    }
                }
                ScrollBar.vertical: ScrollBar {
                    id: scrollBar
                    Timer { id: scrollTimer; interval: 2500; onTriggered: parent.active = false }
                    onActiveChanged: scrollTimer.start()
                }
            }
        }
        Rectangle {
            Layout.fillWidth: true
            Layout.preferredHeight: 116
            color: Style.parameter("color.modalBackground")
            RowLayout {
                anchors {
                    right: parent.right
                    verticalCenter: parent.verticalCenter
                    rightMargin: 24
                }
                spacing: 8
                BasicButton {
                    Layout.preferredHeight: 68
                    Layout.preferredWidth: 168
                    textLayout {
                        fillWidth: true
                        fillHeight: true
                    }
                    horizontalAlignment: Text.AlignLeft
                    verticalAlignment: Text.AlignBottom
                    Style.class: "tmcMultipleChoiceShowTmcInfo"
                    Tr { code: "tmcMultipleChoiceTmcInfo"; defaultMessage: "Информация" }
                    onClicked: {
                        if (selectionModel.hasSelection) {
                            let barcode = proxyModel.data(
                                    proxyModel.index(selectionModel.currentIndex.row, 1))
                            if (handler.showTmcInfo(barcode))
                                accept()
                        }
                    }
                }
                BasicButton {
                    Layout.preferredHeight: 68
                    Layout.preferredWidth: 120
                    textLayout {
                        fillWidth: true
                        fillHeight: true
                    }
                    horizontalAlignment: Text.AlignLeft
                    verticalAlignment: Text.AlignBottom
                    Style.class: "tmcMultipleChoiceCancel"
                    Tr { code: "cancelButton"; defaultMessage: "Отмена" }
                    onClicked: reject()
                }
                BasicButton {
                    Layout.preferredHeight: 68
                    Layout.preferredWidth: 168
                    textLayout {
                        fillWidth: true
                        fillHeight: true
                    }
                    horizontalAlignment: Text.AlignLeft
                    verticalAlignment: Text.AlignBottom
                    Style.class: "tmcMultipleChoiceOk"
                    Tr { code: "selectButton"; defaultMessage: "Выбрать" }
                    onClicked: if (selectionModel.hasSelection) accept()
                }
            }
            // Тень
            Rectangle {
                anchors.bottom: parent.top
                width: parent.width
                height: 5
                gradient: Gradient {
                    orientation: Gradient.Vertical
                    GradientStop { position: 0.0; color: "transparent" }
                    GradientStop { position: 1.0; color: Style.parameter("color.shadow") }
                }
                opacity: 0.3
            }
        }
    }

    footer: BottomPanel { id: bottomPanel }

    // !!!!!!!!!! Клавиатура !!!!!!!!!!
    Popup {
        id: keyboard
        parent: Overlay.overlay
        x: 0
        y: root.height
        width: root.width
        height: root.width * 0.402
        leftPadding: 32
        rightPadding: 32
        topPadding: 16
        bottomPadding: 24
        closePolicy: Popup.NoAutoClose
        contentItem: InlineKeyboard {
            layoutType: "main"
            showCancel: true
            enterTr: Utils.makeTr("undefined", "Выбрать")
            onHide: inputField.focus = false
            onEnter: {
                if (selectionModel.hasSelection) {
                    keyboard.exit = null
                    keyboard.close()
                    accept()
                } else {
                    inputField.focus = false
                }
            }
            onCancel: reject()
        }
        background: Rectangle {
            color: Style.parameter("color.modalBackground")
            // Тень
            Rectangle {
                anchors.bottom: parent.top
                width: parent.width
                height: 5
                gradient: Gradient {
                    orientation: Gradient.Vertical
                    GradientStop { position: 0.0; color: "transparent" }
                    GradientStop { position: 1.0; color: Style.parameter("color.shadow") }
                }
                opacity: 0.3
            }
        }
        Connections {
            target: inputField
            function onFocusChanged() {
                if (inputField.focus)
                    keyboard.open()
                else
                    keyboard.close()
            }
        }
        enter: Transition { SmoothedAnimation { velocity: 2000; property: "y"; to: root.height - keyboard.height - bottomPanel.height } }
        exit: Transition { SmoothedAnimation { velocity: 2000; property: "y"; to: root.height } }
    }

    DialogHandler { id: handler }

    Component.onCompleted: {
        dataModel.prepare(params.type === DialogEnum.ChoiceMultipleSubstring)
        updateModel()
    }

    function accept() {
        let barcode = proxyModel.data(proxyModel.index(selectionModel.currentIndex.row, 1))
        let code = proxyModel.data(proxyModel.index(selectionModel.currentIndex.row, 2))
        handler.sendExtendedInput(barcode, { "code": code, "barcode": barcode }, InputSource.Keyboard)
    }

    function reject() {
        handler.sendExtendedInput("", { "code": "", "barcode": "" }, InputSource.Undefined)
    }

    function updateModel() {
        dataModel.update(nameFilter, barcodeFilter, codeFilter, priceFilter)
    }
}
